﻿Public Module modTopoSort

   ''' <summary> topologische Sortierung nach Lasser </summary>
   Public Function TopoSort(Of T)( _
         ByVal Graph As IEnumerable, ByVal GetSuccessors As Func(Of T, IEnumerable(Of T))) As List(Of T)
      TopoSort = New List(Of T)
      Dim Nodes = Graph.Cast(Of T).ToList
      'assoziiere jeden Node mit einem Zähler der Anzahl seiner Vorgänger
      Dim PreCounts = New Dictionary(Of T, Integer)
      For Each Nd In Nodes
         PreCounts.Add(Nd, 0)
      Next
      For Each Nd In Nodes
         For Each Successor As T In GetSuccessors(Nd)
            PreCounts(Successor) += 1
         Next
      Next
      'in mehrfachen Durchläufen vorgängerfreie Nodes entfernen und ins Ergebnis einreihen
      While Nodes.Count > 0
         For I As Integer = Nodes.Count - 1 To 0 Step -1
            Dim Nd = Nodes(I)
            If PreCounts(Nd) = 0 Then
               For Each Successor As T In GetSuccessors(Nd)
                  PreCounts(Successor) -= 1
               Next
               Nodes.RemoveAt(I)
               TopoSort.Add(Nd)
            End If
         Next
      End While
   End Function

   ''' <summary> topologische Sortierung nach Lasser </summary>
   ''' <remarks>
   ''' Diese Variante geht nicht von den Knoten des Graphen aus, sondern von den Relationen
   ''' </remarks>
   Public Function TopoSort2(Of T)(ByVal Relations As IEnumerable(Of T())) As List(Of T)
      TopoSort2 = New List(Of T)
      Dim Nodes  = Relations.SelectMany(Function(Rel) Rel).Distinct.ToList
      Dim NodeInfos = NewDictionary( _
         From Nd In Nodes _
         Select KeyValue(Nd, New With {.PreCount = 0, .Successors = New List(Of T)}))

      For Each Rel In Relations
         NodeInfos(Rel(0)).Successors.Add(Rel(1))
         NodeInfos(Rel(1)).PreCount += 1
      Next

      While Nodes.Count > 0
         For I As Integer = Nodes.Count - 1 To 0 Step -1
            Dim Nd = Nodes(I)
            Dim TPI = NodeInfos(Nd)
            If TPI.PreCount = 0 Then
               For Each Successor As T In TPI.Successors
                  NodeInfos(Successor).PreCount -= 1
               Next
               Nodes.RemoveAt(I)
               TopoSort2.Add(Nd)
            End If
         Next
      End While
   End Function

End Module
